@@ -10,6 +10,7 @@ from collections import defaultdict
 from itertools import chain
 
 import attr
+import six
 
 from cached_property import cached_property
 
@@ -19,8 +20,12 @@ from .mixins import BasePath
 from ..environment import PYENV_INSTALLED, PYENV_ROOT
 from ..exceptions import InvalidPythonVersion
 from ..utils import (
-    ensure_path, filter_pythons, looks_like_python, optional_instance_of,
-    path_is_known_executable, unnest
+    ensure_path,
+    filter_pythons,
+    looks_like_python,
+    optional_instance_of,
+    path_is_known_executable,
+    unnest,
 )
 from .python import PythonVersion
 
@@ -37,6 +42,7 @@ class SystemPath(object):
     pyenv_finder = attr.ib(default=None, validator=optional_instance_of("PyenvPath"))
     system = attr.ib(default=False)
     _version_dict = attr.ib(default=attr.Factory(defaultdict))
+    ignore_unsupported = attr.ib(default=False)
 
     __finders = attr.ib(default=attr.Factory(dict))
 
@@ -74,7 +80,7 @@ class SystemPath(object):
                     if entry not in self._version_dict[version]:
                         self._version_dict[version].append(entry)
                     continue
-                if isinstance(entry, VersionPath):
+                if type(entry).__name__ == "VersionPath":
                     for path in entry.paths.values():
                         if path not in self._version_dict[version] and path.is_python:
                             self._version_dict[version].append(path)
@@ -129,17 +135,16 @@ class SystemPath(object):
             pyenv_index = self.path_order.index(last_pyenv)
         except ValueError:
             return
-        self.pyenv_finder = PyenvFinder.create(root=PYENV_ROOT)
-        # paths = (v.paths.values() for v in self.pyenv_finder.versions.values())
-        root_paths = (
-            p for path in self.pyenv_finder.expanded_paths for p in path if p.is_root
+        self.pyenv_finder = PyenvFinder.create(
+            root=PYENV_ROOT, ignore_unsupported=self.ignore_unsupported
         )
+        root_paths = [p for p in self.pyenv_finder.roots]
         before_path = self.path_order[: pyenv_index + 1]
         after_path = self.path_order[pyenv_index + 2 :]
         self.path_order = (
-            before_path + [p.path.as_posix() for p in root_paths] + after_path
+            before_path + [p.as_posix() for p in root_paths] + after_path
         )
-        self.paths.update({p.path: p for p in root_paths})
+        self.paths.update(self.pyenv_finder.roots)
         self._register_finder("pyenv", self.pyenv_finder)
 
     def _setup_windows(self):
@@ -154,7 +159,9 @@ class SystemPath(object):
 
     def get_path(self, path):
         path = ensure_path(path)
-        _path = self.paths.get(path.as_posix())
+        _path = self.paths.get(path)
+        if not _path:
+            _path = self.paths.get(path.as_posix())
         if not _path and path.as_posix() in self.path_order:
             _path = PathEntry.create(
                 path=path.absolute(), is_root=True, only_python=self.only_python
@@ -162,6 +169,14 @@ class SystemPath(object):
             self.paths[path.as_posix()] = _path
         return _path
 
+    def _get_paths(self):
+        return (self.get_path(k) for k in self.path_order)
+
+    @cached_property
+    def path_entries(self):
+        paths = self._get_paths()
+        return paths
+
     def find_all(self, executable):
         """Search the path for an executable. Return all copies.
 
@@ -170,8 +185,8 @@ class SystemPath(object):
         :returns: List[PathEntry]
         """
         sub_which = operator.methodcaller("which", name=executable)
-        filtered = filter(None, (sub_which(self.get_path(k)) for k in self.path_order))
-        return [f for f in filtered]
+        filtered = (sub_which(self.get_path(k)) for k in self.path_order)
+        return list(filtered)
 
     def which(self, executable):
         """Search for an executable on the path.
@@ -181,11 +196,39 @@ class SystemPath(object):
         :returns: :class:`~pythonfinder.models.PathEntry` object.
         """
         sub_which = operator.methodcaller("which", name=executable)
-        filtered = filter(None, (sub_which(self.get_path(k)) for k in self.path_order))
-        return next((f for f in filtered), None)
+        filtered = (sub_which(self.get_path(k)) for k in self.path_order)
+        return next(iter(f for f in filtered if f is not None), None)
+
+    def _filter_paths(self, finder):
+        return (
+            pth for pth in unnest(finder(p) for p in self.path_entries if p is not None)
+            if pth is not None
+        )
+
+    def _get_all_pythons(self, finder):
+        paths = {p.path.as_posix(): p for p in self._filter_paths(finder)}
+        paths.update(self.python_executables)
+        return (p for p in paths.values() if p is not None)
+
+    def get_pythons(self, finder):
+        sort_key = operator.attrgetter("as_python.version_sort")
+        return (
+            k for k in sorted(
+                (p for p in self._filter_paths(finder) if p.is_python),
+                key=sort_key,
+                reverse=True
+            ) if k is not None
+        )
 
     def find_all_python_versions(
-        self, major=None, minor=None, patch=None, pre=None, dev=None, arch=None
+        self,
+        major=None,
+        minor=None,
+        patch=None,
+        pre=None,
+        dev=None,
+        arch=None,
+        name=None,
     ):
         """Search for a specific python version on the path. Return all copies
 
@@ -196,32 +239,46 @@ class SystemPath(object):
         :param bool pre: Search for prereleases (default None) - prioritize releases if None
         :param bool dev: Search for devreleases (default None) - prioritize releases if None
         :param str arch: Architecture to include, e.g. '64bit', defaults to None
+        :param str name: The name of a python version, e.g. ``anaconda3-5.3.0``
         :return: A list of :class:`~pythonfinder.models.PathEntry` instances matching the version requested.
         :rtype: List[:class:`~pythonfinder.models.PathEntry`]
         """
 
         sub_finder = operator.methodcaller(
             "find_all_python_versions",
-            major,
+            major=major,
             minor=minor,
             patch=patch,
             pre=pre,
             dev=dev,
             arch=arch,
+            name=name,
         )
+        alternate_sub_finder = None
+        if major and not (minor or patch or pre or dev or arch or name):
+            alternate_sub_finder = operator.methodcaller(
+                "find_all_python_versions",
+                major=None,
+                name=major
+            )
         if os.name == "nt" and self.windows_finder:
             windows_finder_version = sub_finder(self.windows_finder)
             if windows_finder_version:
                 return windows_finder_version
-        paths = (self.get_path(k) for k in self.path_order)
-        path_filter = filter(
-            None, unnest((sub_finder(p) for p in paths if p is not None))
-        )
-        version_sort = operator.attrgetter("as_python.version_sort")
-        return [c for c in sorted(path_filter, key=version_sort, reverse=True)]
+        values = list(self.get_pythons(sub_finder))
+        if not values and alternate_sub_finder is not None:
+            values = list(self.get_pythons(alternate_sub_finder))
+        return values
 
     def find_python_version(
-        self, major=None, minor=None, patch=None, pre=None, dev=None, arch=None
+        self,
+        major=None,
+        minor=None,
+        patch=None,
+        pre=None,
+        dev=None,
+        arch=None,
+        name=None,
     ):
         """Search for a specific python version on the path.
 
@@ -232,10 +289,24 @@ class SystemPath(object):
         :param bool pre: Search for prereleases (default None) - prioritize releases if None
         :param bool dev: Search for devreleases (default None) - prioritize releases if None
         :param str arch: Architecture to include, e.g. '64bit', defaults to None
+        :param str name: The name of a python version, e.g. ``anaconda3-5.3.0``
         :return: A :class:`~pythonfinder.models.PathEntry` instance matching the version requested.
         :rtype: :class:`~pythonfinder.models.PathEntry`
         """
 
+        if isinstance(major, six.string_types) and not minor and not patch:
+            # Only proceed if this is in the format "x.y.z" or similar
+            if major.count(".") > 0 and major[0].isdigit():
+                version = major.split(".", 2)
+                if len(version) > 3:
+                    major, minor, patch, rest = version
+                elif len(version) == 3:
+                    major, minor, patch = version
+                else:
+                    major, minor = version
+            else:
+                name = "{0!s}".format(major)
+                major = None
         sub_finder = operator.methodcaller(
             "find_python_version",
             major,
@@ -244,7 +315,15 @@ class SystemPath(object):
             pre=pre,
             dev=dev,
             arch=arch,
+            name=name,
         )
+        alternate_sub_finder = None
+        if major and not (minor or patch or pre or dev or arch or name):
+            alternate_sub_finder = operator.methodcaller(
+                "find_all_python_versions",
+                major=None,
+                name=major
+            )
         if major and minor and patch:
             _tuple_pre = pre if pre is not None else False
             _tuple_dev = dev if dev is not None else False
@@ -254,35 +333,41 @@ class SystemPath(object):
             windows_finder_version = sub_finder(self.windows_finder)
             if windows_finder_version:
                 return windows_finder_version
-        paths = (self.get_path(k) for k in self.path_order)
-        path_filter = filter(None, (sub_finder(p) for p in paths if p is not None))
-        version_sort = operator.attrgetter("as_python.version_sort")
-        ver = next(
-            (c for c in sorted(path_filter, key=version_sort, reverse=True)), None
-        )
+        ver = next(iter(self.get_pythons(sub_finder)), None)
+        if not ver and alternate_sub_finder is not None:
+            ver = next(iter(self.get_pythons(alternate_sub_finder)), None)
         if ver:
             if ver.as_python.version_tuple[:5] in self.python_version_dict:
                 self.python_version_dict[ver.as_python.version_tuple[:5]].append(ver)
             else:
                 self.python_version_dict[ver.as_python.version_tuple[:5]] = [ver]
+            print(ver)
         return ver
 
     @classmethod
-    def create(cls, path=None, system=False, only_python=False, global_search=True):
+    def create(
+        cls,
+        path=None,
+        system=False,
+        only_python=False,
+        global_search=True,
+        ignore_unsupported=True,
+    ):
         """Create a new :class:`pythonfinder.models.SystemPath` instance.
 
         :param path: Search path to prepend when searching, defaults to None
         :param path: str, optional
-        :param system: Whether to use the running python by default instead of searching, defaults to False
-        :param system: bool, optional
-        :param only_python: Whether to search only for python executables, defaults to False
-        :param only_python: bool, optional
+        :param bool system: Whether to use the running python by default instead of searching, defaults to False
+        :param bool only_python: Whether to search only for python executables, defaults to False
+        :param bool ignore_unsupported: Whether to ignore unsupported python versions, if False, an error is raised, defaults to True
         :return: A new :class:`pythonfinder.models.SystemPath` instance.
         :rtype: :class:`pythonfinder.models.SystemPath`
         """
 
         path_entries = defaultdict(PathEntry)
         paths = []
+        if ignore_unsupported:
+            os.environ["PYTHONFINDER_IGNORE_UNSUPPORTED"] = fs_str("1")
         if global_search:
             paths = os.environ.get("PATH").split(os.pathsep)
         if path:
@@ -303,6 +388,7 @@ class SystemPath(object):
             only_python=only_python,
             system=system,
             global_search=global_search,
+            ignore_unsupported=ignore_unsupported,
         )
 
 
@@ -312,7 +398,8 @@ class PathEntry(BasePath):
     _children = attr.ib(default=attr.Factory(dict))
     is_root = attr.ib(default=True)
     only_python = attr.ib(default=False)
-    py_version = attr.ib(default=None)
+    name = attr.ib()
+    py_version = attr.ib()
     pythons = attr.ib()
 
     def __str__(self):
@@ -325,17 +412,46 @@ class PathEntry(BasePath):
             children = self.path.iterdir()
         return children
 
+    def _gen_children(self):
+        pass_name = self.name != self.path.name
+        pass_args = {"is_root": False, "only_python": self.only_python}
+        if pass_name:
+            pass_args["name"] = self.name
+
+        if not self.is_dir:
+            yield (self.path.as_posix(), copy.deepcopy(self))
+        elif self.is_root:
+            for child in self._filter_children():
+                yield (child.as_posix(), PathEntry.create(path=child, **pass_args))
+        return
+
     @cached_property
     def children(self):
-        if not self._children and self.is_dir and self.is_root:
-            self._children = {
-                child.as_posix(): PathEntry.create(path=child, is_root=False)
-                for child in self._filter_children()
-            }
-        elif not self.is_dir:
-            self._children = {self.path.as_posix(): self}
+        if not self._children:
+            children = {}
+            for child_key, child_val in self._gen_children():
+                children[child_key] = child_val
+            self._children = children
         return self._children
 
+    @name.default
+    def get_name(self):
+        return self.path.name
+
+    @py_version.default
+    def get_py_version(self):
+        from ..environment import IGNORE_UNSUPPORTED
+        if self.is_dir:
+            return None
+        if self.is_python:
+            from .python import PythonVersion
+            try:
+                py_version = PythonVersion.from_path(path=self, name=self.name)
+            except InvalidPythonVersion:
+                py_version = None
+            return py_version
+        return
+
     @pythons.default
     def get_pythons(self):
         pythons = defaultdict()
@@ -347,56 +463,62 @@ class PathEntry(BasePath):
         else:
             if self.is_python:
                 _path = ensure_path(self.path)
-                pythons[_path.as_posix()] = copy.deepcopy(self)
+                pythons[_path.as_posix()] = self
         return pythons
 
     @cached_property
     def as_python(self):
+        py_version = None
+        if self.py_version:
+            return self.py_version
         if not self.is_dir and self.is_python:
-            if not self.py_version:
-                try:
-                    from .python import PythonVersion
-
-                    self.py_version = PythonVersion.from_path(self.path)
-                except (ValueError, InvalidPythonVersion):
-                    self.py_version = None
-        return self.py_version
+            try:
+                from .python import PythonVersion
+                py_version = PythonVersion.from_path(path=attr.evolve(self), name=self.name)
+            except (ValueError, InvalidPythonVersion):
+                py_version = None
+        return py_version
 
     @classmethod
-    def create(cls, path, is_root=False, only_python=False, pythons=None):
+    def create(cls, path, is_root=False, only_python=False, pythons=None, name=None):
         """Helper method for creating new :class:`pythonfinder.models.PathEntry` instances.
 
-        :param path: Path to the specified location.
-        :type path: str
-        :param is_root: Whether this is a root from the environment PATH variable, defaults to False
-        :param is_root: bool, optional
-        :param only_python: Whether to search only for python executables, defaults to False
-        :param only_python: bool, optional
-        :param pythons: A dictionary of existing python objects (usually from a finder), defaults to None
-        :param pythons: dict, optional
+        :param str path: Path to the specified location.
+        :param bool is_root: Whether this is a root from the environment PATH variable, defaults to False
+        :param bool only_python: Whether to search only for python executables, defaults to False
+        :param dict pythons: A dictionary of existing python objects (usually from a finder), defaults to None
+        :param str name: Name of the python version, e.g. ``anaconda3-5.3.0``
         :return: A new instance of the class.
         :rtype: :class:`pythonfinder.models.PathEntry`
         """
 
         target = ensure_path(path)
-        creation_args = {"path": target, "is_root": is_root, "only_python": only_python}
+        guessed_name = False
+        if not name:
+            guessed_name = True
+            name = target.name
+        creation_args = {"path": target, "is_root": is_root, "only_python": only_python, "name": name}
         if pythons:
             creation_args["pythons"] = pythons
         _new = cls(**creation_args)
         if pythons and only_python:
             children = {}
+            child_creation_args = {
+                "is_root": False,
+                "py_version": python,
+                "only_python": only_python
+            }
+            if not guessed_name:
+                child_creation_args["name"] = name
             for pth, python in pythons.items():
                 pth = ensure_path(pth)
                 children[pth.as_posix()] = PathEntry(
-                    path=pth, is_root=False, only_python=only_python, py_version=python
+                    path=pth,
+                    **child_creation_args
                 )
             _new._children = children
         return _new
 
-    @cached_property
-    def name(self):
-        return self.path.name
-
     @cached_property
     def is_dir(self):
         try:
@@ -412,28 +534,5 @@ class PathEntry(BasePath):
     @cached_property
     def is_python(self):
         return self.is_executable and (
-            self.py_version or looks_like_python(self.path.name)
+            looks_like_python(self.path.name)
         )
-
-
-@attr.s
-class VersionPath(SystemPath):
-    base = attr.ib(default=None, validator=optional_instance_of(Path))
-
-    @classmethod
-    def create(cls, path, only_python=True, pythons=None):
-        """Accepts a path to a base python version directory.
-
-        Generates the pyenv version listings for it"""
-        path = ensure_path(path)
-        path_entries = defaultdict(PathEntry)
-        if not path.name.lower() in ["scripts", "bin"]:
-            bin_name = "Scripts" if os.name == "nt" else "bin"
-            bin_dir = path / bin_name
-        else:
-            bin_dir = path
-        current_entry = PathEntry.create(
-            bin_dir, is_root=True, only_python=True, pythons=pythons
-        )
-        path_entries[bin_dir.as_posix()] = current_entry
-        return cls(base=bin_dir, paths=path_entries)
