@@ -7,27 +7,30 @@ from .models import SystemPath
 
 
 class Finder(object):
-    def __init__(self, path=None, system=False, global_search=True):
+    def __init__(self, path=None, system=False, global_search=True, ignore_unsupported=True):
         """Finder A cross-platform Finder for locating python and other executables.
-        
+
         Searches for python and other specified binaries starting in `path`, if supplied,
         but searching the bin path of `sys.executable` if `system=True`, and then
         searching in the `os.environ['PATH']` if `global_search=True`.  When `global_search`
-        is `False`, this search operation is restricted to the allowed locations of 
+        is `False`, this search operation is restricted to the allowed locations of
         `path` and `system`.
-        
+
         :param path: A bin-directory search location, defaults to None
         :param path: str, optional
         :param system: Whether to include the bin-dir of `sys.executable`, defaults to False
         :param system: bool, optional
         :param global_search: Whether to search the global path from os.environ, defaults to True
         :param global_search: bool, optional
+        :param ignore_unsupported: Whether to ignore unsupported python versions, if False, an error is raised, defaults to True
+        :param ignore_unsupported: bool, optional
         :returns: a :class:`~pythonfinder.pythonfinder.Finder` object.
         """
 
         self.path_prepend = path
         self.global_search = global_search
         self.system = system
+        self.ignore_unsupported = ignore_unsupported
         self._system_path = None
         self._windows_finder = None
 
@@ -38,6 +41,7 @@ class Finder(object):
                 path=self.path_prepend,
                 system=self.system,
                 global_search=self.global_search,
+                ignore_unsupported=self.ignore_unsupported,
             )
         return self._system_path
 
@@ -53,7 +57,7 @@ class Finder(object):
         return self.system_path.which(exe)
 
     def find_python_version(
-        self, major, minor=None, patch=None, pre=None, dev=None, arch=None
+        self, major=None, minor=None, patch=None, pre=None, dev=None, arch=None, name=None
     ):
         from .models import PythonVersion
 
@@ -65,12 +69,24 @@ class Finder(object):
             and patch is None
         ):
             if arch is None and "-" in major:
-                major, arch = major.rsplit("-", 1)
-                if not arch.isnumeric():
-                    major = "{0}-{1}".format(major, arch)
+                orig_string = "{0!s}".format(major)
+                major, _, arch = major.rpartition("-")
+                if arch.startswith("x"):
+                    arch = arch.lstrip("x")
+                if arch.lower().endswith("bit"):
+                    arch = arch.lower().replace("bit", "")
+                if not (arch.isdigit() and (int(arch) & int(arch) - 1) == 0):
+                    major = orig_string
+                    arch = None
                 else:
                     arch = "{0}bit".format(arch)
-            version_dict = PythonVersion.parse(major)
+            try:
+                version_dict = PythonVersion.parse(major)
+            except ValueError:
+                if name is None:
+                    name = "{0!s}".format(major)
+                    major = None
+                version_dict = {}
             major = version_dict.get("major", major)
             minor = version_dict.get("minor", minor)
             patch = version_dict.get("patch", patch)
@@ -79,16 +95,16 @@ class Finder(object):
             arch = version_dict.get("architecture", arch) if arch is None else arch
         if os.name == "nt":
             match = self.windows_finder.find_python_version(
-                major, minor=minor, patch=patch, pre=pre, dev=dev, arch=arch
+                major=major, minor=minor, patch=patch, pre=pre, dev=dev, arch=arch, name=name
             )
             if match:
                 return match
         return self.system_path.find_python_version(
-            major=major, minor=minor, patch=patch, pre=pre, dev=dev, arch=arch
+            major=major, minor=minor, patch=patch, pre=pre, dev=dev, arch=arch, name=name
         )
 
     def find_all_python_versions(
-        self, major=None, minor=None, patch=None, pre=None, dev=None, arch=None
+        self, major=None, minor=None, patch=None, pre=None, dev=None, arch=None, name=None
     ):
         version_sort = operator.attrgetter("as_python.version_sort")
         python_version_dict = getattr(self.system_path, "python_version_dict")
@@ -105,7 +121,7 @@ class Finder(object):
             paths = sorted(paths, key=version_sort, reverse=True)
             return paths
         versions = self.system_path.find_all_python_versions(
-            major=major, minor=minor, patch=patch, pre=pre, dev=dev, arch=arch
+            major=major, minor=minor, patch=patch, pre=pre, dev=dev, arch=arch, name=name
         )
         if not isinstance(versions, list):
             versions = [versions]
