@@ -14,29 +14,47 @@ import six
 
 import vistir
 
-from vistir.compat import Path
-
 from .exceptions import InvalidPythonVersion
 
+try:
+    from functools import lru_cache
+except ImportError:
+    from backports.functools_lru_cache import lru_cache
 
-PYTHON_IMPLEMENTATIONS = ("python", "ironpython", "jython", "pypy")
+
+PYTHON_IMPLEMENTATIONS = (
+    "python", "ironpython", "jython", "pypy", "anaconda", "miniconda",
+    "stackless", "activepython", "micropython"
+)
+RULES_BASE = ["*{0}", "*{0}?", "*{0}?.?", "*{0}?.?m"]
+RULES = [rule.format(impl) for impl in PYTHON_IMPLEMENTATIONS for rule in RULES_BASE]
 
 KNOWN_EXTS = {"exe", "py", "fish", "sh", ""}
 KNOWN_EXTS = KNOWN_EXTS | set(
     filter(None, os.environ.get("PATHEXT", "").split(os.pathsep))
 )
 
+MATCH_RULES = []
+for rule in RULES:
+    MATCH_RULES.extend(
+        [
+            "{0}.{1}".format(rule, ext) if ext else "{0}".format(rule)
+            for ext in KNOWN_EXTS
+        ]
+    )
 
+
+@lru_cache(maxsize=128)
 def get_python_version(path):
     """Get python version string using subprocess from a given path."""
     version_cmd = [path, "-c", "import sys; print(sys.version.split()[0])"]
     try:
-        out, _ = vistir.misc.run(version_cmd)
+        out, _ = vistir.misc.run(version_cmd, block=True, nospin=True)
     except OSError:
         raise InvalidPythonVersion("%s is not a valid python path" % path)
     if not out:
         raise InvalidPythonVersion("%s is not a valid python path" % path)
-    return out
+    return out.strip()
 
 
 def optional_instance_of(cls):
@@ -44,13 +62,14 @@ def optional_instance_of(cls):
 
 
 def path_and_exists(path):
-    return attr.validators.instance_of(Path) and path.exists()
+    return attr.validators.instance_of(vistir.compat.Path) and path.exists()
 
 
 def path_is_executable(path):
     return os.access(str(path), os.X_OK)
 
 
+@lru_cache(maxsize=1024)
 def path_is_known_executable(path):
     return (
         path_is_executable(path)
@@ -59,25 +78,19 @@ def path_is_known_executable(path):
     )
 
 
+@lru_cache(maxsize=1024)
 def looks_like_python(name):
-    rules = ["*python", "*python?", "*python?.?", "*python?.?m"]
-    match_rules = []
-    for rule in rules:
-        match_rules.extend(
-            [
-                "{0}.{1}".format(rule, ext) if ext else "{0}".format(rule)
-                for ext in KNOWN_EXTS
-            ]
-        )
     if not any(name.lower().startswith(py_name) for py_name in PYTHON_IMPLEMENTATIONS):
         return False
-    return any(fnmatch(name, rule) for rule in match_rules)
+    return any(fnmatch(name, rule) for rule in MATCH_RULES)
 
 
+@lru_cache(maxsize=128)
 def path_is_python(path):
     return path_is_executable(path) and looks_like_python(path.name)
 
 
+@lru_cache(maxsize=1024)
 def ensure_path(path):
     """Given a path (either a string or a Path object), expand variables and return a Path object.
 
@@ -87,14 +100,10 @@ def ensure_path(path):
     :rtype: :class:`~pathlib.Path`
     """
 
-    if isinstance(path, Path):
-        path = path.as_posix()
-    path = Path(os.path.expandvars(path))
-    try:
-        path = path.resolve()
-    except OSError:
-        path = path.absolute()
-    return path
+    if isinstance(path, vistir.compat.Path):
+        return path
+    path = vistir.compat.Path(os.path.expandvars(path))
+    return path.absolute()
 
 
 def _filter_none(k, v):
@@ -103,10 +112,11 @@ def _filter_none(k, v):
     return False
 
 
+@lru_cache(maxsize=128)
 def filter_pythons(path):
     """Return all valid pythons in a given path"""
-    if not isinstance(path, Path):
-        path = Path(str(path))
+    if not isinstance(path, vistir.compat.Path):
+        path = vistir.compat.Path(str(path))
     if not path.is_dir():
         return path if path_is_python(path) else None
     return filter(lambda x: path_is_python(x), path.iterdir())
